/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.parser.software;

import java.io.InputStream;

import net.sf.jannot.Entry;
import net.sf.jannot.EntrySet;
import net.sf.jannot.Feature;
import net.sf.jannot.Location;
import net.sf.jannot.MemoryFeatureAnnotation;
import net.sf.jannot.Strand;
import net.sf.jannot.Type;
import net.sf.jannot.parser.Parser;
import be.abeel.io.LineIterator;

/**
 * @author Thomas Abeel
 * 
 */
public class FindPeaksParser extends Parser {

	/**
	 * @param dataKey
	 */
	public FindPeaksParser() {
		super(null);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.sf.jannot.parser.Parser#parse(java.io.InputStream,
	 * net.sf.jannot.source.DataSource, net.sf.jannot.EntrySet)
	 */
	@Override
	public EntrySet parse(InputStream is, EntrySet set) {
		if (set == null)
			set = new EntrySet();

		LineIterator it = new LineIterator(is);
		it.setSkipBlanks(true);
		it.setSkipComments(true);
		/* Skip first line */
		it.next();
		Type t = Type.get("peak");
		for (String line : it) {
			String[] arr = line.split("[ \t]+");
			Entry e = set.getOrCreateEntry(arr[1]);
			Feature f = new Feature();
			int start = Integer.parseInt(arr[2]);
			int end = Integer.parseInt(arr[3]);
			f.setType(t);
			f.addQualifier("max_coord", arr[4]);
			f.addQualifier("Name", "Peak "+arr[0]);
			//f.addQualifier("t-score", arr[5]);
			f.setScore(Double.parseDouble(arr[5]));
			f.addLocation(new Location(start, end));
			f.setStrand(Strand.UNKNOWN);
			MemoryFeatureAnnotation fa = e.getMemoryAnnotation(t);
			fa.add(f);

		}
		return set;
	}

}
