/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2017 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.jannot.source.cache;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.net.URL;

import net.sf.jannot.EntrySet;
import net.sf.jannot.exception.ReadFailedException;
import net.sf.jannot.exception.SaveFailedException;
import net.sf.jannot.parser.Parser;
import net.sf.jannot.source.SSL;
import net.sf.jannot.source.URLSource;

public class CachedURLSource extends URLSource {

	public CachedURLSource(URL url) throws IOException {
		super(url, null);

	}

	

	@Override
	public EntrySet read(EntrySet set) throws ReadFailedException {
		if (!SourceCache.contains(url)) {
			SSL.certify(url);
			try {
				super.setParser(Parser.detectParser(url.openStream(),url));
				final PipedInputStream in = new PipedInputStream();
				final PipedOutputStream forParser = new PipedOutputStream(in);

				new Thread(new Runnable() {
					public void run() {
						try {
							OutputStream out = SourceCache.startCaching(url);
							InputStream is=url.openStream();
							byte[] buffer = new byte[100000];
							while (true) {
								
								int amountRead = is.read(buffer);
								if (amountRead == -1) {
									break;
								}
								forParser.write(buffer, 0, amountRead);
								out.write(buffer, 0, amountRead);

							}
							forParser.close();
							out.close();
							SourceCache.finish(url);
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}

					}
				}).start();

				super.setIos(in);
			} catch (Exception e) {
				throw new ReadFailedException(e);

			}
			return super.read(set);
		} else {
			try {
				return SourceCache.get(url).read(set);
			} catch (IOException e) {
				throw new ReadFailedException(e);
			}
		}
	}

}
